using System;
using System.Windows.Forms;	// for DialogResult etc.

namespace MyNamespace
{
	/// <summary>
	///		BBallCS - Baseball probability in Visual C# .net, 32-Bit Windows
	///		Developed in TURBO Pascal 5.5 and 6.0, converted to Visual Basic 6
	///		and now in Visual C# .Net
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	/// This program is free software; you can redistribute it and/or modify
	/// it under the terms of the GNU General Public License (GPL) as published
	/// by the Free Software Foundation. cf. the file COPYING.txt.
	///
	/// ------------ *** end of legal notice *** ------------------ 
	/// </summary>
	public class BBallCS
	{
		public BBallCS()
		{
			// no constructor logic
		}

		public static void ComputeProb()
		// Compute probability using a bivariate binomial distribution as a model
		{
			double a;
			double b;
			try
			{
				pError = false;
				double[] e = new double[eachOther + 1];	// Binomial coefficients, games to play each other
				double[] s = new double[eachOther + 1];	// Running sum of 2 * e[i]
				double[] f;								// B. C., games not played with each other
				double[] g;								// Sum of 2 * e[i] for 2nd place team win, e[i] for tie
				if(magicNTie1 >= 0)
				{
					f = new double[magicNTie1 + 1];
					g = new double[magicNTie1 + 1];
				}
				else
				{
					f = new double[1];
					g = new double[1];
				}
				a = gamesLeft1 + gamesLeft2 - eachOther - eachOther;	// a = not played with each other games
				b = 1;
				f[0] = 1;
				for (int i = 1; i <= magicNTie1; i++)		// Compute binomial coefficients
				{
					f[i] = f[i - 1] * (a / b);
					if (Double.IsPositiveInfinity(f[i])) throw(new OverflowException());
					a--;
					b++;
				}
				a = eachOther;
				b = 1;
				e[0] = 1;
				s[0] = 2;
				for (int i = 1; i <= eachOther; i++)		// Compute binomial coefficients
				{
					e[i] = e[i - 1] * (a / b);
					if (Double.IsPositiveInfinity(e[i])) throw(new OverflowException());
					a--;
					b++;
					s[i] = s[i - 1] + 2 * e[i];
					if (Double.IsPositiveInfinity(s[i])) throw(new OverflowException());
				}
				for (int i = 0; i <= magicNTie1; i++)		// Compute g[i]
				{
					int j = (int)(magicNTie1 - i) / 2;
					if (j <= eachOther)
					{
						g[i] = s[j];
						if ((j + j) == (magicNTie1 - i))
							g[i] -= e[j];		// Adjust for tie
					}
					else
					{
						g[i] = s[eachOther];
					}
				}
				q = 0;
				for (int i = 0; i <= magicNTie1; i++)		// Compute probability that 2nd place team beats 1st place team
					q += f[i] * g[i];
				a = gamesLeft1 + gamesLeft2 - eachOther + 1;
				b = Math.Pow(2, a);				// b = 2 ** Flips
				q /= b;
				p = 1 - q;
			}
			catch (Exception ex)
			{
				string message = "\"" + ex.Message + "\"\n\n" +
						"Input numbers are too large to compute binomial coefficients!";
				DialogResult MsgButton = MessageBox.Show(message,	// Display message
					Global.title,
					MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				// ----------
				p = 0;
				q = 0;
				pError = true;
			}
		} // ComputeProb

		public static void ExpandCase() // Compute related data
		{
			int i;
			ahead2 = (long)(2 * ahead); // Round
			i = (int)(gamesLeft1 + gamesLeft2 - ahead2);
			magicNTie1 = (i / 2); magicNWin1 = magicNTie1 + 1;
			magicNTie2 = gamesLeft1 + gamesLeft2 - magicNTie1; magicNWin2 = magicNTie2 + 1;
			i = (int)(gamesLeft1 + gamesLeft2 + ahead2);
			if ((i & 1) != 0) // if i is odd
				aheadError = true;
			else
				aheadError = false;
		}

		public static bool aheadError;		// True if ahead value not possible
		public static bool eachOtherError;	// True if GE value not possible
		public static bool pError;			// True if Overflow while computing P
		public static double ahead;			// Games 1st place team is ahead. 0, 0.5, ...
		public static double ahead2;		// Twice games ahead = 2 * ahead, 0, 1, ...
		public static double p;				// Probability that 1st place team beats 2nd place team
		public static double q;				// Probability that 2nd place team beats 1st place team
		public static int eachOther;		// Games to play each other
		public static int gamesLeft1;		// Games Left to play, 1st place team
		public static int gamesLeft2;		// Games Left to play, 2nd place team
		public static long magicNTie1;		// Magic Number to tie for 1st place team
		public static long magicNTie2;		// Magic Number to tie for 2nd place team
		public static long magicNWin1;		// Magic Number to win for 1st place team
		public static long magicNWin2;		// Magic Number to win for 2nd place team
		public static string teamName1;		// Team's name, 1st place team
		public static string teamName2;		// Team's name, 2nd place team
	}
}
